bool module_african PARAMS ((struct recode_outer *));
bool module_afrtran PARAMS ((struct recode_outer *));
bool module_applemac PARAMS ((struct recode_outer *));
bool module_atarist PARAMS ((struct recode_outer *));
bool module_bangbang PARAMS ((struct recode_outer *));
bool module_cdcnos PARAMS ((struct recode_outer *));
bool module_ebcdic PARAMS ((struct recode_outer *));
bool module_ibmpc PARAMS ((struct recode_outer *));
bool module_iconqnx PARAMS ((struct recode_outer *));
bool module_latin1_ascii PARAMS ((struct recode_outer *));
bool module_mule PARAMS ((struct recode_outer *));
bool module_strips PARAMS ((struct recode_outer *));
bool module_testdump PARAMS ((struct recode_outer *));
bool module_ucs PARAMS ((struct recode_outer *));
bool module_utf16 PARAMS ((struct recode_outer *));
bool module_utf7 PARAMS ((struct recode_outer *));
bool module_utf8 PARAMS ((struct recode_outer *));
bool module_varia PARAMS ((struct recode_outer *));
bool module_vietnamese PARAMS ((struct recode_outer *));
bool module_flat PARAMS ((struct recode_outer *));
bool module_html PARAMS ((struct recode_outer *));
bool module_latin1_latex PARAMS ((struct recode_outer *));
bool module_latin1_texte PARAMS ((struct recode_outer *));
bool module_rfc1345 PARAMS ((struct recode_outer *));
bool module_texinfo PARAMS ((struct recode_outer *));
bool module_base64 PARAMS ((struct recode_outer *));
bool module_dump PARAMS ((struct recode_outer *));
bool module_endline PARAMS ((struct recode_outer *));
bool module_permutations PARAMS ((struct recode_outer *));
bool module_quoted_printable PARAMS ((struct recode_outer *));
bool module_ascii_latin1 PARAMS ((struct recode_outer *));
bool module_latex_latin1 PARAMS ((struct recode_outer *));
bool module_texte_latin1 PARAMS ((struct recode_outer *));
void delmodule_african PARAMS ((struct recode_outer *));
void delmodule_afrtran PARAMS ((struct recode_outer *));
void delmodule_applemac PARAMS ((struct recode_outer *));
void delmodule_atarist PARAMS ((struct recode_outer *));
void delmodule_bangbang PARAMS ((struct recode_outer *));
void delmodule_cdcnos PARAMS ((struct recode_outer *));
void delmodule_ebcdic PARAMS ((struct recode_outer *));
void delmodule_ibmpc PARAMS ((struct recode_outer *));
void delmodule_iconqnx PARAMS ((struct recode_outer *));
void delmodule_latin1_ascii PARAMS ((struct recode_outer *));
void delmodule_mule PARAMS ((struct recode_outer *));
void delmodule_strips PARAMS ((struct recode_outer *));
void delmodule_testdump PARAMS ((struct recode_outer *));
void delmodule_ucs PARAMS ((struct recode_outer *));
void delmodule_utf16 PARAMS ((struct recode_outer *));
void delmodule_utf7 PARAMS ((struct recode_outer *));
void delmodule_utf8 PARAMS ((struct recode_outer *));
void delmodule_varia PARAMS ((struct recode_outer *));
void delmodule_vietnamese PARAMS ((struct recode_outer *));
void delmodule_flat PARAMS ((struct recode_outer *));
void delmodule_html PARAMS ((struct recode_outer *));
void delmodule_latin1_latex PARAMS ((struct recode_outer *));
void delmodule_latin1_texte PARAMS ((struct recode_outer *));
void delmodule_rfc1345 PARAMS ((struct recode_outer *));
void delmodule_texinfo PARAMS ((struct recode_outer *));
void delmodule_base64 PARAMS ((struct recode_outer *));
void delmodule_dump PARAMS ((struct recode_outer *));
void delmodule_endline PARAMS ((struct recode_outer *));
void delmodule_permutations PARAMS ((struct recode_outer *));
void delmodule_quoted_printable PARAMS ((struct recode_outer *));
void delmodule_ascii_latin1 PARAMS ((struct recode_outer *));
void delmodule_latex_latin1 PARAMS ((struct recode_outer *));
void delmodule_texte_latin1 PARAMS ((struct recode_outer *));
