/* Special definitions for GNU recode, processed by autoheader.
   Copyright © 1993, 1998, 1999, 2000 Free Software Foundation, Inc.
   François Pinard <pinard@iro.umontreal.ca>, 1993.
*/

/* Default charset is `Latin-1' on UNIX, `IBM-PC' on MSDOS.  */
#undef DEFAULT_CHARSET

/* Define to the name of the distribution.  */
#undef PACKAGE

/* Define to 1 if ANSI function prototypes are usable.  */
#undef PROTOTYPES

/* Define to the version of the distribution.  */
#undef VERSION

/* Define to 1 for better use of the debugging malloc library.  See
   site ftp.antaire.com in antaire/src, file dmalloc/dmalloc.tar.gz.  */
#undef WITH_DMALLOC

/* Define to 1 if NLS is requested.  */
#undef ENABLE_NLS

/* Define as 1 if you have dcgettext.  */
#undef HAVE_DCGETTEXT

/* Define as 1 if you have gettext and don't want to use GNU gettext.  */
#undef HAVE_GETTEXT

/* Define if your locale.h file contains LC_MESSAGES.  */
#undef HAVE_LC_MESSAGES

/* Define to 1 if stpcpy function is available.  */
#undef HAVE_STPCPY

/* Define to the installation directory for locales.  */
#undef LOCALEDIR

/* Define to 1 when also using Bruno Haible `libiconv' library.  */
#undef WITH_LIBICONV

/* Define to rpl_malloc if the replacement function should be used.  */
#undef malloc

/* Define to rpl_realloc if the replacement function should be used.  */
#undef realloc
