/* A lexical scanner generated by flex */

/* Scanner skeleton version:
 * $Header: /home/daffy/u0/vern/flex/RCS/flex.skl,v 2.91 96/09/10 16:58:48 vern Exp $
 */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5

#include <stdio.h>


/* cfront 1.2 defines "c_plusplus" instead of "__cplusplus" */
#ifdef c_plusplus
#ifndef __cplusplus
#define __cplusplus
#endif
#endif


#ifdef __cplusplus

#include <stdlib.h>
#include <unistd.h>

/* Use prototypes in function declarations. */
#define YY_USE_PROTOS

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

#if __STDC__

#define YY_USE_PROTOS
#define YY_USE_CONST

#endif	/* __STDC__ */
#endif	/* ! __cplusplus */

#ifdef __TURBOC__
 #pragma warn -rch
 #pragma warn -use
#include <io.h>
#include <stdlib.h>
#define YY_USE_CONST
#define YY_USE_PROTOS
#endif

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif


#ifdef YY_USE_PROTOS
#define YY_PROTO(proto) proto
#else
#define YY_PROTO(proto) ()
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN yy_start = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START ((yy_start - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE yyrestart( yyin )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#define YY_BUF_SIZE 16384

typedef struct yy_buffer_state *YY_BUFFER_STATE;

extern int yyleng;
extern FILE *yyin, *yyout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

/* The funky do-while in the following #define is used to turn the definition
 * int a single C statement (which needs a semi-colon terminator).  This
 * avoids problems with code like:
 *
 * 	if ( condition_holds )
 *		yyless( 5 );
 *	else
 *		do_something_else();
 *
 * Prior to using the do-while the compiler would get upset at the
 * "else" because it interpreted the "if" statement as being all
 * done when it reached the ';' after the yyless() call.
 */

/* Return all but the first 'n' matched characters back to the input stream. */

#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
		*yy_cp = yy_hold_char; \
		YY_RESTORE_YY_MORE_OFFSET \
		yy_c_buf_p = yy_cp = yy_bp + n - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, yytext_ptr )

/* The following is because we cannot portably get our hands on size_t
 * (without autoconf's help, which isn't available because we want
 * flex-generated scanners to compile on their own).
 */
typedef unsigned int yy_size_t;


struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;
#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via yyrestart()), so that the user can continue scanning by
	 * just pointing yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2
	};

static YY_BUFFER_STATE yy_current_buffer = 0;

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 */
#define YY_CURRENT_BUFFER yy_current_buffer


/* yy_hold_char holds the character lost when yytext is formed. */
static char yy_hold_char;

static int yy_n_chars;		/* number of characters read into yy_ch_buf */


int yyleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 1;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow yywrap()'s to do buffer switches
 * instead of setting up a fresh yyin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void yyrestart YY_PROTO(( FILE *input_file ));

void yy_switch_to_buffer YY_PROTO(( YY_BUFFER_STATE new_buffer ));
void yy_load_buffer_state YY_PROTO(( void ));
YY_BUFFER_STATE yy_create_buffer YY_PROTO(( FILE *file, int size ));
void yy_delete_buffer YY_PROTO(( YY_BUFFER_STATE b ));
void yy_init_buffer YY_PROTO(( YY_BUFFER_STATE b, FILE *file ));
void yy_flush_buffer YY_PROTO(( YY_BUFFER_STATE b ));
#define YY_FLUSH_BUFFER yy_flush_buffer( yy_current_buffer )

YY_BUFFER_STATE yy_scan_buffer YY_PROTO(( char *base, yy_size_t size ));
YY_BUFFER_STATE yy_scan_string YY_PROTO(( yyconst char *yy_str ));
YY_BUFFER_STATE yy_scan_bytes YY_PROTO(( yyconst char *bytes, int len ));

static void *yy_flex_alloc YY_PROTO(( yy_size_t ));
static void *yy_flex_realloc YY_PROTO(( void *, yy_size_t ));
static void yy_flex_free YY_PROTO(( void * ));

#define yy_new_buffer yy_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! yy_current_buffer ) \
		yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE ); \
	yy_current_buffer->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! yy_current_buffer ) \
		yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE ); \
	yy_current_buffer->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (yy_current_buffer->yy_at_bol)


#define yywrap() 1
#define YY_SKIP_YYWRAP
typedef unsigned char YY_CHAR;
FILE *yyin = (FILE *) 0, *yyout = (FILE *) 0;
typedef int yy_state_type;
extern char *yytext;
#define yytext_ptr yytext

static yy_state_type yy_get_previous_state YY_PROTO(( void ));
static yy_state_type yy_try_NUL_trans YY_PROTO(( yy_state_type current_state ));
static int yy_get_next_buffer YY_PROTO(( void ));
static void yy_fatal_error YY_PROTO(( yyconst char msg[] ));

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
	yytext_ptr = yy_bp; \
	yyleng = (int) (yy_cp - yy_bp); \
	yy_hold_char = *yy_cp; \
	*yy_cp = '\0'; \
	yy_c_buf_p = yy_cp;

#define YY_NUM_RULES 208
#define YY_END_OF_BUFFER 209
static yyconst short int yy_accept[598] =
    {   0,
        0,    0,    0,    0,    0,    0,    0,    0,  209,  208,
      208,  208,  208,  208,  208,  208,  208,  208,  208,  208,
      208,  208,  208,  208,  208,  208,  208,  208,  208,  208,
      208,  208,  208,  208,  208,  208,  208,  208,  208,  208,
      208,   65,  208,  208,  208,  208,  208,  208,  208,  208,
      208,  208,  208,  208,  208,  208,  208,  208,  208,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,   66,   83,   87,   69,   57,
       58,   59,   60,   78,   81,   73,  147,  147,  110,   68,

        0,   61,    0,  147,  147,  147,  147,  147,  147,  139,
      147,  147,   62,   63,    0,   71,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,  149,    0,  151,  150,
        0,    0,  161,  162,  158,    0,  171,  170,    0,    0,
      177,  176,    0,  183,  182,  148,    0,  154,  153,    0,
        0,  167,  168,  164,    0,  174,  173,    0,  180,  179,
        0,  186,  185,    1,    2,    7,   12,   16,   22,   27,
       34,   39,   43,   49,   29,   54,   56,    4,   10,   14,
       19,   25,   28,   31,   37,   41,   46,   52,   55,    8,
       35,   23,   50,    5,    3,    6,   11,    9,   15,   13,

       17,   20,   18,   21,   26,   24,   32,   38,   42,   47,
       53,   30,   36,   40,   45,   51,   33,   44,   48,   92,
       99,  103,  109,  114,    0,  121,  128,  138,  143,  145,
        0,   89,   97,  101,  106,  112,  115,    0,  118,  126,
      135,  141,  144,    0,  147,   93,   94,    0,  110,    0,
       68,    0,   90,   98,  102,  107,  113,    0,  119,  127,
      136,  142,    0,   88,   96,  100,  105,  111,    0,  117,
      125,  134,  140,    0,  122,  123,  147,  147,  147,  147,
        0,  147,  147,  147,  139,    0,  147,  116,   91,  104,
      108,  120,  133,  137,    0,    0,    0,    0,    0,    0,

        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
      152,  156,  163,  160,  172,    0,  178,  184,  155,  157,
      169,  166,  175,  181,  187,  132,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,  130,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
       93,    0,   94,    0,  147,  110,   68,  131,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,  129,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,  122,
        0,  123,    0,  147,   95,  124,  147,  147,    0,    0,
      147,  147,   72,  139,  147,  116,    0,    0,    0,    0,

        0,    0,    0,  196,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,  159,    0,  165,  132,  146,    0,    0,  130,    0,
        0,   93,   94,  131,    0,    0,  129,    0,    0,  122,
      123,  147,  147,  147,  147,  147,   72,    0,  147,  116,
        0,    0,    0,    0,    0,    0,    0,    0,    0,  198,
        0,    0,  202,    0,    0,    0,    0,    0,  201,  146,
        0,  132,  132,  130,  130,  131,  131,  129,  129,  147,
       80,  147,    0,    0,    0,   72,   67,    0,    0,    0,
        0,    0,    0,    0,  207,    0,    0,    0,    0,    0,

        0,  204,    0,  146,  147,  147,    0,    0,    0,   67,
        0,  190,    0,    0,  206,  189,    0,    0,    0,    0,
      200,    0,    0,    0,  205,  147,  147,   85,   84,   86,
        0,    0,   67,    0,    0,  197,    0,  193,    0,    0,
        0,    0,  147,  147,   85,    0,   84,    0,   86,    0,
        0,    0,    0,    0,    0,    0,  191,    0,    0,  199,
      203,    0,    0,   64,   70,   85,   84,   86,    0,    0,
        0,    0,    0,    0,  188,  192,  194,  195,   64,    0,
       70,    0,    0,    0,   82,   76,   77,    0,   64,   70,
       79,   75,    0,    0,    0,   74,    0

    } ;

static yyconst int yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    2,    3,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    3,    4,    5,    6,    7,    8,    9,   10,    1,
        1,    1,    1,   11,   12,    1,   13,    1,   14,   15,
       16,   17,    1,    1,    1,    1,    1,   18,    1,   19,
        1,   20,   21,    1,   22,   23,   24,   25,   26,   25,
       27,   28,   29,   30,   25,   30,   25,   31,   32,   25,
       25,   25,   33,   34,   35,   25,   25,   25,   36,   25,
        1,   37,    1,   38,   39,   40,   41,   42,   43,   44,

       45,   46,   47,   48,   49,   30,   50,   51,   52,   53,
       54,   55,   25,   56,   57,   58,   59,   25,   25,   60,
       61,   25,   62,    1,   63,   64,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst int yy_meta[65] =
    {   0,
        1,    1,    2,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    1,    1,    1,    1,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    2,    1,    1
    } ;

static yyconst short int yy_base[611] =
    {   0,
        0,    0,    0,    0,    2,   23,   87,    0,  943,  944,
      940,  939,  938,  937,  936,  935,  934,  933,  932,  931,
      930,  929,  928,  927,  926,  925,  924,  923,  922,  921,
      920,  919,  918,  917,  916,  915,  876,  905,  874,  147,
      873,  944,  190,  902,   10,  900,  216,   29,  900,  222,
      223,    0,  869,  231,  897,  240,  237,  241,  246,  268,
      308,   13,   14,  902,  901,  286,  894,  301,  318,  840,
      341,  326,  893,  384,  422,  337,  891,  350,  358,  837,
      382,  895,  392,    4,  251,  944,  944,  944,  460,  944,
      944,  944,  944,  501,  944,  944,  250,    0,   13,   14,

      542,  944,  583,  254,  181,  329,   15,  332,  316,   18,
      212,   22,  944,  944,  396,  944,    3,  854,   24,   25,
      844,  856,  843,  211,  112,  835,  944,    0,  944,  944,
      382,    0,  884,  944,  944,    0,  944,  944,   29,    0,
      944,  944,    0,  944,  944,  944,    0,  944,  944,  420,
        0,  883,  944,  944,    0,  944,  944,    0,  944,  944,
        0,  944,  944,  944,  944,  944,  944,  944,  944,  944,
      944,  944,  944,  944,  944,  944,  944,  944,  944,  944,
      944,  944,  944,  944,  944,  944,  944,  944,  944,  944,
      944,  944,  944,  944,  944,  944,  944,  944,  944,  944,

      944,  944,  944,  944,  944,  944,  944,  944,  944,  944,
      944,  944,  944,  944,  944,  944,  944,  944,  944,  944,
      944,  944,  944,  944,  843,  944,  944,  944,  944,  944,
      624,  944,  944,  944,  944,  944,  944,  842,  944,  944,
      944,  944,  944,  664,  158,  227,  290,  827,  318,  826,
      345,  825,  944,  944,  944,  944,  944,  838,  944,  944,
      944,  944,  704,  944,  944,  944,  944,  944,  837,  944,
      944,  944,  944,  742,  362,  369,  383,  397,  406,  432,
      410,  436,  408,  444,  453,  822,  466,  462,  944,  944,
      944,  944,  944,  944,  512,  470,  842,  826,  829,  834,

      827,  838,  829,  828,  827,  821,   34,   16,  833,  823,
      944,  944,  944,  862,  944,  236,  944,  944,  944,  944,
      944,  861,  944,  944,  944,  469,  807,  806,  805,  804,
      803,  816,  801,  800,  799,  798,  797,  477,  796,  795,
      794,  793,  792,  791,  804,  789,  788,  787,  786,  785,
      487,  784,  490,  783,  944,  944,  944,  496,  782,  781,
      780,  779,  778,  791,  776,  775,  774,  773,  497,  772,
      771,  770,  769,  768,  781,  766,  765,  764,  763,  499,
      762,  505,  761,  507,  944,  944,  523,  529,  760,  759,
      508,  513,  510,  944,  544,  526,  758,  757,  756,  755,

      754,  753,  752,  944,  755,  764,  771,  757,  751,  260,
      750,  753,  296,  297,  740,  315,  739,  756,  754,  745,
      752,  944,  366,  944,  532,  537,  729,  728,  538,  727,
      726,  944,  944,  545,  723,  722,  551,  721,  719,  944,
      944,  573,  577,  592,  600,  579,  581,  718,  589,  944,
      733,  729,  726,  728,  727,  729,  718,  371,  707,  944,
      715,  713,  944,  705,  711,  710,  394,  707,  944,  590,
      692,  944,  944,  944,  944,  944,  944,  944,  944,  619,
      595,  614,  741,  740,  352,  944,  605,  478,  693,  704,
      502,  584,  699,  700,  944,  699,  702,  605,  701,  693,

      691,  944,  616,  944,  633,  626,  147,  711,  130,  622,
      641,  944,  637,  621,  944,  944,  622,  616,  626,  622,
      944,  577,  560,  552,  944,  635,  646,  644,  652,  655,
      330,  697,  944,  659,  541,  944,  533,  944,  661,  684,
      511,  505,  684,  657,  665,  482,  669,  455,  688,  449,
      450,  452,  496,  478,  477,  435,  944,  698,  702,  944,
      944,  717,  719,  695,  707,  944,  944,  944,  459,  457,
      345,  340,  330,  321,  944,  944,  944,  944,  718,  282,
      726,  275,  247,  245,  944,  944,  944,  242,  944,  944,
      944,  944,  225,  260,  181,  944,  944,  801,  803,  226,

      222,  162,  160,   78,   71,   27,    5,    4,    1,  805
    } ;

static yyconst short int yy_def[611] =
    {   0,
      598,  598,  598,    3,  598,  598,  597,    7,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,   43,  597,  597,  597,  597,  597,  597,
      597,   43,   43,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  599,  599,  599,  599,

      597,  597,  597,  599,  599,  599,  599,  599,  599,  599,
      599,  599,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  600,  597,  597,
      597,  601,  597,  597,  597,  602,  597,  597,  597,  603,
      597,  597,  604,  597,  597,  597,  605,  597,  597,  597,
      606,  597,  597,  597,  607,  597,  597,  608,  597,  597,
      609,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,

      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  599,  599,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  599,  599,  599,  597,  599,  599,
      597,  599,  599,  599,  597,  597,  599,  599,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,

      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  610,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  610,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  610,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  610,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  599,  597,  597,  599,  599,  597,  597,
      599,  599,  599,  597,  599,  597,  597,  597,  597,  597,

      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  610,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  599,  599,  599,  599,  599,  597,  597,  599,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  599,
      599,  599,  597,  597,  597,  597,  599,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,

      597,  597,  597,  597,  599,  599,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  599,  599,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  599,  599,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  599,  599,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,    0,  597,  597,  597,

      597,  597,  597,  597,  597,  597,  597,  597,  597,  597
    } ;

static yyconst short int yy_nxt[1009] =
    {   0,
      597,  597,  245,  325,   11,   37,  324,  323,  177,   12,
       13,   38,   14,  189,  128,  249,  251,  245,   15,   16,
      285,   17,   39,   18,  245,   19,   37,  128,   20,  321,
       21,   22,   38,  136,   23,   24,  190,   25,   40,   26,
       27,   41,   28,   39,   29,  192,  136,  129,   30,  130,
      316,  296,   31,   32,  297,  191,   33,  418,   34,   40,
       35,  248,   41,   36,  299,   42,  137,  193,  138,  316,
      282,  419,  300,  319,  250,  252,  248,  301,  288,  286,
      318,  302,  416,  248,  303,  417,   42,   43,   43,   43,
       43,   43,   43,   43,   43,   43,   44,   43,   43,   43,

       43,   43,   43,   43,   43,   43,   43,   43,   45,   10,
       46,   10,   47,   10,   10,   48,   49,   10,   50,   10,
       49,   51,   10,   43,   52,   43,   53,   54,   10,   55,
       10,   56,   10,   10,   10,   57,   10,   49,   10,   10,
       58,   10,   10,   10,   49,   59,   10,   10,   43,   43,
       43,   89,   90,   91,   92,   93,   94,   95,   96,  309,
      245,  528,  317,  529,  315,  310,  531,  532,   97,   98,
       98,   98,   98,   98,   98,   98,   98,   98,   99,  100,
       98,   98,   98,  245,  101,  102,  103,  104,  105,  106,
       98,   98,  107,   98,   98,   98,   98,   98,  108,  109,

      110,  111,   98,  112,   98,   98,   98,   98,  113,  114,
      115,  117,  118,  119,  245,  120,  121,  122,  123,  248,
      132,  277,  124,  125,  313,  133,  140,  143,  311,  351,
      117,  118,  119,  132,  126,  147,  121,  122,  123,  140,
      143,  155,  248,  596,  151,  158,  124,  125,  147,  152,
      161,  307,  245,  134,  155,  135,  245,  151,  158,  141,
      144,  142,  145,  161,  423,  287,  595,  594,  148,  308,
      149,  246,  196,  248,  156,  247,  157,  153,  159,  154,
      160,  201,  204,  162,  423,  163,  164,  165,  352,  166,
      166,  217,  353,  167,  275,  178,  168,  593,  276,  169,

      460,  461,  170,  218,  219,  167,  456,  592,  171,  591,
      179,  248,  172,  460,  461,  248,  173,  457,  245,  463,
      249,  174,  168,  194,  175,  195,  176,  180,  177,  178,
      170,  278,  463,  179,  245,  182,  180,  590,  197,  181,
      198,  171,  182,  183,  589,  169,  184,  251,  184,  196,
      181,  354,  185,  192,  172,  199,  186,  200,  509,  185,
      284,  187,  173,  205,  380,  206,  188,  186,  189,  588,
      469,  382,  279,  283,  207,  495,  212,  248,  202,  250,
      203,  551,  280,  469,  552,  245,  174,  208,  495,  213,
      281,  187,  587,  248,  193,  209,  176,  214,  502,  278,

      217,  188,  586,  312,  204,  194,  252,  585,  245,  197,
      245,  502,  199,  312,  355,  202,  312,  289,  205,  210,
      385,  215,  312,  381,  207,  384,  290,  291,  208,  211,
      383,  216,  209,  389,  245,  312,  292,  210,  245,  386,
      312,  320,  211,  195,  248,  219,  245,  198,  293,  294,
      200,  320,  390,  203,  320,  285,  206,  295,  248,  387,
      320,  392,  212,  584,  396,  583,  213,  248,  245,  248,
      214,  425,  355,  320,  404,  215,  391,  574,  320,  429,
      216,  220,  512,  573,  572,  221,  388,  404,  222,  351,
      393,  223,  353,  248,  224,  512,  225,  248,  434,  437,

      226,  380,  571,  570,  227,  248,  515,  382,  569,  245,
      245,  568,  447,  228,  286,  245,  405,  567,  229,  515,
      230,  231,  232,  397,  395,  245,  233,  248,  396,  234,
      427,  245,  235,  398,  425,  236,  237,  238,  430,  470,
      429,  239,  399,  400,  566,  240,  245,  434,  352,  563,
      445,  354,  401,  437,  241,  562,  442,  435,  438,  242,
      381,  243,  244,  253,  402,  403,  383,  254,  248,  248,
      255,  448,  446,  256,  248,  245,  257,  559,  258,  245,
      443,  245,  259,  447,  248,  558,  260,  397,  516,  444,
      248,  245,  470,  427,  245,  261,  449,  245,  471,  430,

      262,  516,  245,  263,  264,  248,  435,  510,  265,  521,
      542,  266,  438,  483,  267,  484,  245,  268,  541,  269,
      525,  245,  521,  270,  510,  540,  536,  271,  245,  480,
      538,  481,  487,  525,  248,  245,  272,  245,  248,  536,
      485,  273,  448,  538,  274,  327,  545,  482,  245,  328,
      248,  471,  329,  248,  547,  330,  248,  549,  331,  245,
      332,  248,  506,  557,  333,  560,  511,  545,  334,  505,
      539,  547,  527,  526,  537,  248,  557,  335,  560,  535,
      248,  534,  336,  511,  337,  339,  245,  248,  561,  340,
      549,  543,  341,  544,  248,  342,  248,  579,  343,  344,

      345,  561,  575,  533,  346,  546,  576,  248,  347,  581,
      553,  554,  555,  548,  565,  575,  550,  348,  248,  576,
      579,  577,  349,  578,  350,  359,  546,  530,  581,  360,
      548,  564,  361,  556,  577,  362,  578,  524,  363,  523,
      364,  522,  520,  519,  365,  248,  518,  517,  366,  550,
      514,  513,  508,  507,  504,  503,  580,  367,  501,  500,
      499,  498,  368,  370,  497,  496,  494,  371,  582,  493,
      372,  492,  491,  373,  490,  489,  374,  488,  375,  580,
      486,  479,  376,  478,  477,  476,  377,  582,  475,  474,
      473,  472,  468,  467,  466,  378,  465,  464,  462,  459,

      379,   10,   10,   10,   98,   98,  426,  426,  458,  455,
      454,  453,  452,  451,  294,  293,  292,  291,  290,  289,
      450,  386,  385,  441,  440,  273,  272,  271,  270,  439,
      268,  267,  266,  265,  264,  262,  261,  260,  259,  436,
      257,  256,  255,  254,  253,  433,  432,  243,  242,  241,
      240,  239,  431,  237,  236,  235,  234,  233,  232,  230,
      229,  228,  227,  226,  428,  224,  223,  222,  221,  220,
      424,  422,  421,  420,  415,  414,  413,  412,  411,  410,
      409,  408,  407,  406,  394,  369,  358,  357,  356,  355,
      338,  326,  322,  314,  303,  306,  305,  304,  298,  175,

      218,  191,  183,  201,  190,  165,  164,  150,  146,  139,
      131,  127,  116,   88,   87,   86,   85,   84,   83,   82,
       81,   80,   79,   78,   77,   76,   75,   74,   73,   72,
       71,   70,   69,   68,   67,   66,   65,   64,   63,   62,
       61,   60,  597,    9,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,

      597,  597,  597,  597,  597,  597,  597,  597
    } ;

static yyconst short int yy_chk[1009] =
    {   0,
        0,    0,   98,  609,    3,    5,  608,  607,   84,    3,
        3,    5,    3,   84,   45,   99,  100,  107,    3,    3,
      110,    3,    5,    3,  112,    3,    6,   45,    3,  606,
        3,    3,    6,   48,    3,    3,   62,    3,    5,    3,
        3,    5,    3,    6,    3,   63,   48,   45,    3,   45,
      139,  117,    3,    3,  117,   62,    3,  308,    3,    6,
        3,   98,    6,    3,  119,    5,   48,   63,   48,  139,
      107,  308,  119,  605,   99,  100,  107,  119,  112,  110,
      604,  120,  307,  112,  120,  307,    6,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,

        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,   40,   40,   40,   40,   40,   40,   40,   40,  125,
      245,  507,  603,  507,  602,  125,  509,  509,   40,   40,
       40,   40,   40,   40,   40,   40,   40,   40,   40,   40,
       40,   40,   40,  105,   40,   40,   40,   40,   40,   40,
       40,   40,   40,   40,   40,   40,   40,   40,   40,   40,

       40,   40,   40,   40,   40,   40,   40,   40,   40,   40,
       40,   43,   43,   43,  111,   43,   43,   43,   43,  245,
       47,  105,   43,   43,  601,   47,   50,   51,  600,  246,
       43,   43,   43,   47,   43,   54,   43,   43,   43,   50,
       51,   57,  105,  595,   56,   58,   43,   43,   54,   56,
       59,  124,   97,   47,   57,   47,  104,   56,   58,   50,
       51,   50,   51,   59,  316,  111,  594,  593,   54,  124,
       54,   97,   85,  111,   57,   97,   57,   56,   58,   56,
       58,   85,   85,   59,  316,   59,   60,   60,  246,   60,
       66,   85,  247,   60,  104,   66,   60,  588,  104,   60,

      413,  414,   60,   85,   85,   68,  410,  584,   60,  583,
       68,   97,   60,  413,  414,  104,   60,  410,  109,  416,
      249,   60,   69,   66,   60,   66,   60,   69,   60,   61,
       72,  106,  416,   61,  108,   72,   61,  582,   68,   61,
       68,   76,   61,   61,  580,   71,   76,  251,   61,   66,
       71,  247,   61,   71,   78,   69,   61,   69,  485,   78,
      109,   61,   79,   72,  275,   72,   61,   79,   61,  574,
      423,  276,  106,  108,   76,  458,   76,  109,   71,  249,
       71,  531,  106,  423,  531,  277,   81,   78,  458,   78,
      106,   81,  573,  108,   81,   79,   83,   79,  467,  278,

       76,   83,  572,  131,   71,   74,  251,  571,  279,   74,
      283,  467,   74,  131,  485,   74,  131,  115,   74,   81,
      278,   81,  131,  275,   74,  277,  115,  115,   74,   83,
      276,   83,   74,  281,  280,  131,  115,   74,  282,  278,
      131,  150,   74,   75,  277,   81,  284,   75,  115,  115,
       75,  150,  281,   75,  150,  285,   75,  115,  278,  279,
      150,  283,   75,  570,  288,  569,   75,  279,  287,  283,
       75,  326,  281,  150,  296,   75,  282,  556,  150,  338,
       75,   89,  488,  555,  554,   89,  280,  296,   89,  351,
      284,   89,  353,  280,   89,  488,   89,  282,  358,  369,

       89,  380,  553,  552,   89,  284,  491,  382,  551,  384,
      391,  550,  393,   89,  285,  392,  296,  548,   89,  491,
       89,   89,   94,  288,  287,  387,   94,  287,  396,   94,
      326,  388,   94,  295,  425,   94,   94,   94,  338,  426,
      429,   94,  295,  295,  546,   94,  395,  434,  351,  542,
      391,  353,  295,  437,   94,  541,  384,  358,  369,   94,
      380,   94,   94,  101,  295,  295,  382,  101,  384,  391,
      101,  393,  392,  101,  392,  442,  101,  537,  101,  443,
      387,  446,  101,  447,  387,  535,  101,  396,  492,  388,
      388,  449,  470,  425,  444,  101,  395,  481,  426,  429,

      101,  492,  445,  101,  103,  395,  434,  487,  103,  498,
      524,  103,  437,  445,  103,  445,  482,  103,  523,  103,
      503,  480,  498,  103,  510,  522,  517,  103,  506,  442,
      519,  443,  449,  503,  442,  505,  103,  526,  443,  517,
      446,  103,  447,  519,  103,  231,  528,  444,  527,  231,
      449,  470,  231,  444,  529,  231,  481,  530,  231,  544,
      231,  445,  482,  534,  231,  539,  487,  545,  231,  480,
      520,  547,  506,  505,  518,  482,  534,  231,  539,  514,
      480,  513,  231,  510,  231,  244,  543,  506,  540,  244,
      549,  526,  244,  527,  505,  244,  526,  564,  244,  244,

      244,  540,  558,  511,  244,  528,  559,  527,  244,  565,
      532,  532,  532,  529,  544,  558,  530,  244,  544,  559,
      579,  562,  244,  563,  244,  263,  545,  508,  581,  263,
      547,  543,  263,  532,  562,  263,  563,  501,  263,  500,
      263,  499,  497,  496,  263,  543,  494,  493,  263,  549,
      490,  489,  484,  483,  471,  468,  564,  263,  466,  465,
      464,  462,  263,  274,  461,  459,  457,  274,  565,  456,
      274,  455,  454,  274,  453,  452,  274,  451,  274,  579,
      448,  439,  274,  438,  436,  435,  274,  581,  431,  430,
      428,  427,  421,  420,  419,  274,  418,  417,  415,  412,

      274,  598,  598,  598,  599,  599,  610,  610,  411,  409,
      408,  407,  406,  405,  403,  402,  401,  400,  399,  398,
      397,  390,  389,  383,  381,  379,  378,  377,  376,  375,
      374,  373,  372,  371,  370,  368,  367,  366,  365,  364,
      363,  362,  361,  360,  359,  354,  352,  350,  349,  348,
      347,  346,  345,  344,  343,  342,  341,  340,  339,  337,
      336,  335,  334,  333,  332,  331,  330,  329,  328,  327,
      322,  314,  310,  309,  306,  305,  304,  303,  302,  301,
      300,  299,  298,  297,  286,  269,  258,  252,  250,  248,
      238,  225,  152,  133,  126,  123,  122,  121,  118,   82,

       80,   77,   73,   70,   67,   65,   64,   55,   53,   49,
       46,   44,   41,   39,   38,   37,   36,   35,   34,   33,
       32,   31,   30,   29,   28,   27,   26,   25,   24,   23,
       22,   21,   20,   19,   18,   17,   16,   15,   14,   13,
       12,   11,    9,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,
      597,  597,  597,  597,  597,  597,  597,  597,  597,  597,

      597,  597,  597,  597,  597,  597,  597,  597
    } ;

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *yytext;
#define INITIAL 0
/* This file is generated automatically by `mergelex.py'.  */
#define YY_NO_UNPUT
#include "common.h"
static RECODE_CONST_REQUEST request;
static RECODE_SUBTASK subtask;
#define ascii_latin1 1

#define latex_latin1 2

void texte_latin1_diaeresis PARAMS ((void));
#define texte_latin1 3


/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int yywrap YY_PROTO(( void ));
#else
extern int yywrap YY_PROTO(( void ));
#endif
#endif

#ifndef YY_NO_UNPUT
static void yyunput YY_PROTO(( int c, char *buf_ptr ));
#endif

#ifndef yytext_ptr
static void yy_flex_strncpy YY_PROTO(( char *, yyconst char *, int ));
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen YY_PROTO(( yyconst char * ));
#endif

#ifndef YY_NO_INPUT
#ifdef __cplusplus
static int yyinput YY_PROTO(( void ));
#else
static int input YY_PROTO(( void ));
#endif
#endif

#if YY_STACK_USED
static int yy_start_stack_ptr = 0;
static int yy_start_stack_depth = 0;
static int *yy_start_stack = 0;
#ifndef YY_NO_PUSH_STATE
static void yy_push_state YY_PROTO(( int new_state ));
#endif
#ifndef YY_NO_POP_STATE
static void yy_pop_state YY_PROTO(( void ));
#endif
#ifndef YY_NO_TOP_STATE
static int yy_top_state YY_PROTO(( void ));
#endif

#else
#define YY_NO_PUSH_STATE 1
#define YY_NO_POP_STATE 1
#define YY_NO_TOP_STATE 1
#endif

#ifdef YY_MALLOC_DECL
YY_MALLOC_DECL
#else
#if __STDC__
#ifndef __cplusplus
#include <stdlib.h>
#endif
#else
/* Just try to get by without declaring the routines.  This will fail
 * miserably on non-ANSI systems for which sizeof(size_t) != sizeof(int)
 * or sizeof(void*) != sizeof(int).
 */
#endif
#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */

#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO (void) fwrite( yytext, yyleng, 1, yyout )
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( yy_current_buffer->yy_is_interactive ) \
		{ \
		int c = '*', n; \
		for ( n = 0; n < max_size && \
			     (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( yyin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else if ( ((result = fread( buf, 1, max_size, yyin )) == 0) \
		  && ferror( yyin ) ) \
		YY_FATAL_ERROR( "input in flex scanner failed" );
#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL int yylex YY_PROTO(( void ))
#endif

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	YY_USER_ACTION

YY_DECL
	{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;



	if ( yy_init )
		{
		yy_init = 0;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! yy_start )
			yy_start = 1;	/* first start state */

		if ( ! yyin )
			yyin = stdin;

		if ( ! yyout )
			yyout = stdout;

		if ( ! yy_current_buffer )
			yy_current_buffer =
				yy_create_buffer( yyin, YY_BUF_SIZE );

		yy_load_buffer_state();
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		yy_cp = yy_c_buf_p;

		/* Support of yytext. */
		*yy_cp = yy_hold_char;

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = yy_start;
yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			if ( yy_accept[yy_current_state] )
				{
				yy_last_accepting_state = yy_current_state;
				yy_last_accepting_cpos = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 598 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 944 );

yy_find_action:
		yy_act = yy_accept[yy_current_state];
		if ( yy_act == 0 )
			{ /* have to back up */
			yy_cp = yy_last_accepting_cpos;
			yy_current_state = yy_last_accepting_state;
			yy_act = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;


do_action:	/* This label is used only to access EOF actions. */


		switch ( yy_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = yy_hold_char;
			yy_cp = yy_last_accepting_cpos;
			yy_current_state = yy_last_accepting_state;
			goto yy_find_action;

case YY_STATE_EOF(INITIAL):
case YY_STATE_EOF(ascii_latin1):
case YY_STATE_EOF(latex_latin1):
case YY_STATE_EOF(texte_latin1):
{ return 1; }
	YY_BREAK

case 1:
YY_RULE_SETUP
{ put_byte (171, subtask); }
	YY_BREAK
case 2:
YY_RULE_SETUP
{ put_byte (187, subtask); }
	YY_BREAK
case 3:
YY_RULE_SETUP
{ put_byte (192, subtask); }
	YY_BREAK
case 4:
YY_RULE_SETUP
{ put_byte (193, subtask); }
	YY_BREAK
case 5:
YY_RULE_SETUP
{ put_byte (194, subtask); }
	YY_BREAK
case 6:
YY_RULE_SETUP
{ put_byte (195, subtask); }
	YY_BREAK
case 7:
YY_RULE_SETUP
{ put_byte (196, subtask); }
	YY_BREAK
case 8:
YY_RULE_SETUP
{ put_byte (199, subtask); }
	YY_BREAK
case 9:
YY_RULE_SETUP
{ put_byte (200, subtask); }
	YY_BREAK
case 10:
YY_RULE_SETUP
{ put_byte (201, subtask); }
	YY_BREAK
case 11:
YY_RULE_SETUP
{ put_byte (202, subtask); }
	YY_BREAK
case 12:
YY_RULE_SETUP
{ put_byte (203, subtask); }
	YY_BREAK
case 13:
YY_RULE_SETUP
{ put_byte (204, subtask); }
	YY_BREAK
case 14:
YY_RULE_SETUP
{ put_byte (205, subtask); }
	YY_BREAK
case 15:
YY_RULE_SETUP
{ put_byte (206, subtask); }
	YY_BREAK
case 16:
YY_RULE_SETUP
{ put_byte (207, subtask); }
	YY_BREAK
case 17:
YY_RULE_SETUP
{ put_byte (209, subtask); }
	YY_BREAK
case 18:
YY_RULE_SETUP
{ put_byte (210, subtask); }
	YY_BREAK
case 19:
YY_RULE_SETUP
{ put_byte (211, subtask); }
	YY_BREAK
case 20:
YY_RULE_SETUP
{ put_byte (212, subtask); }
	YY_BREAK
case 21:
YY_RULE_SETUP
{ put_byte (213, subtask); }
	YY_BREAK
case 22:
YY_RULE_SETUP
{ put_byte (214, subtask); }
	YY_BREAK
case 23:
YY_RULE_SETUP
{ put_byte (216, subtask); }
	YY_BREAK
case 24:
YY_RULE_SETUP
{ put_byte (217, subtask); }
	YY_BREAK
case 25:
YY_RULE_SETUP
{ put_byte (218, subtask); }
	YY_BREAK
case 26:
YY_RULE_SETUP
{ put_byte (219, subtask); }
	YY_BREAK
case 27:
YY_RULE_SETUP
{ put_byte (220, subtask); }
	YY_BREAK
case 28:
YY_RULE_SETUP
{ put_byte (221, subtask); }
	YY_BREAK
case 29:
YY_RULE_SETUP
{ put_byte (223, subtask); }
	YY_BREAK
case 30:
YY_RULE_SETUP
{ put_byte (224, subtask); }
	YY_BREAK
case 31:
YY_RULE_SETUP
{ put_byte (225, subtask); }
	YY_BREAK
case 32:
YY_RULE_SETUP
{ put_byte (226, subtask); }
	YY_BREAK
case 33:
YY_RULE_SETUP
{ put_byte (227, subtask); }
	YY_BREAK
case 34:
YY_RULE_SETUP
{ put_byte (228, subtask); }
	YY_BREAK
case 35:
YY_RULE_SETUP
{ put_byte (231, subtask); }
	YY_BREAK
case 36:
YY_RULE_SETUP
{ put_byte (232, subtask); }
	YY_BREAK
case 37:
YY_RULE_SETUP
{ put_byte (233, subtask); }
	YY_BREAK
case 38:
YY_RULE_SETUP
{ put_byte (234, subtask); }
	YY_BREAK
case 39:
YY_RULE_SETUP
{ put_byte (235, subtask); }
	YY_BREAK
case 40:
YY_RULE_SETUP
{ put_byte (236, subtask); }
	YY_BREAK
case 41:
YY_RULE_SETUP
{ put_byte (237, subtask); }
	YY_BREAK
case 42:
YY_RULE_SETUP
{ put_byte (238, subtask); }
	YY_BREAK
case 43:
YY_RULE_SETUP
{ put_byte (239, subtask); }
	YY_BREAK
case 44:
YY_RULE_SETUP
{ put_byte (241, subtask); }
	YY_BREAK
case 45:
YY_RULE_SETUP
{ put_byte (242, subtask); }
	YY_BREAK
case 46:
YY_RULE_SETUP
{ put_byte (243, subtask); }
	YY_BREAK
case 47:
YY_RULE_SETUP
{ put_byte (244, subtask); }
	YY_BREAK
case 48:
YY_RULE_SETUP
{ put_byte (245, subtask); }
	YY_BREAK
case 49:
YY_RULE_SETUP
{ put_byte (246, subtask); }
	YY_BREAK
case 50:
YY_RULE_SETUP
{ put_byte (248, subtask); }
	YY_BREAK
case 51:
YY_RULE_SETUP
{ put_byte (249, subtask); }
	YY_BREAK
case 52:
YY_RULE_SETUP
{ put_byte (250, subtask); }
	YY_BREAK
case 53:
YY_RULE_SETUP
{ put_byte (251, subtask); }
	YY_BREAK
case 54:
YY_RULE_SETUP
{ put_byte (252, subtask); }
	YY_BREAK
case 55:
YY_RULE_SETUP
{ put_byte (253, subtask); }
	YY_BREAK
case 56:
YY_RULE_SETUP
{ put_byte (255, subtask); }
	YY_BREAK


case 57:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE ('#', subtask); }
	YY_BREAK
case 58:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE ('$', subtask); }
	YY_BREAK
case 59:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE ('%', subtask); }
	YY_BREAK
case 60:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE ('&', subtask); }
	YY_BREAK
case 61:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE ('_', subtask); }
	YY_BREAK
case 62:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE ('{', subtask); }
	YY_BREAK
case 63:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE ('}', subtask); }
	YY_BREAK
case 64:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE ('\\', subtask); }
	YY_BREAK
case 65:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (160, subtask); }
	YY_BREAK
case 66:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (161, subtask); }
	YY_BREAK
case 67:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (163, subtask); }
	YY_BREAK
case 68:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (167, subtask); }
	YY_BREAK
case 69:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (168, subtask); }
	YY_BREAK
case 70:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (169, subtask); }
	YY_BREAK
case 71:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (171, subtask); }
	YY_BREAK
case 72:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (172, subtask); }
	YY_BREAK
case 73:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (173, subtask); }
	YY_BREAK
case 74:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (176, subtask); }
	YY_BREAK
case 75:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (177, subtask); }
	YY_BREAK
case 76:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (178, subtask); }
	YY_BREAK
case 77:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (179, subtask); }
	YY_BREAK
case 78:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (180, subtask); }
	YY_BREAK
case 79:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (181, subtask); }
	YY_BREAK
case 80:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (183, subtask); }
	YY_BREAK
case 81:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (184, subtask); }
	YY_BREAK
case 82:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (185, subtask); }
	YY_BREAK
case 83:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (187, subtask); }
	YY_BREAK
case 84:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (188, subtask); }
	YY_BREAK
case 85:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (189, subtask); }
	YY_BREAK
case 86:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (190, subtask); }
	YY_BREAK
case 87:
YY_RULE_SETUP
{ PUT_NON_DIACRITIC_BYTE (191, subtask); }
	YY_BREAK
case 88:
YY_RULE_SETUP
{ put_byte (192, subtask); }
	YY_BREAK
case 89:
YY_RULE_SETUP
{ put_byte (193, subtask); }
	YY_BREAK
case 90:
YY_RULE_SETUP
{ put_byte (194, subtask); }
	YY_BREAK
case 91:
YY_RULE_SETUP
{ put_byte (195, subtask); }
	YY_BREAK
case 92:
YY_RULE_SETUP
{ put_byte (196, subtask); }
	YY_BREAK
case 93:
YY_RULE_SETUP
{ put_byte (197, subtask); }
	YY_BREAK
case 94:
YY_RULE_SETUP
{ put_byte (198, subtask); }
	YY_BREAK
case 95:
YY_RULE_SETUP
{ put_byte (199, subtask); }
	YY_BREAK
case 96:
YY_RULE_SETUP
{ put_byte (200, subtask); }
	YY_BREAK
case 97:
YY_RULE_SETUP
{ put_byte (201, subtask); }
	YY_BREAK
case 98:
YY_RULE_SETUP
{ put_byte (202, subtask); }
	YY_BREAK
case 99:
YY_RULE_SETUP
{ put_byte (203, subtask); }
	YY_BREAK
case 100:
YY_RULE_SETUP
{ put_byte (204, subtask); }
	YY_BREAK
case 101:
YY_RULE_SETUP
{ put_byte (205, subtask); }
	YY_BREAK
case 102:
YY_RULE_SETUP
{ put_byte (206, subtask); }
	YY_BREAK
case 103:
YY_RULE_SETUP
{ put_byte (207, subtask); }
	YY_BREAK
case 104:
YY_RULE_SETUP
{ put_byte (209, subtask); }
	YY_BREAK
case 105:
YY_RULE_SETUP
{ put_byte (210, subtask); }
	YY_BREAK
case 106:
YY_RULE_SETUP
{ put_byte (211, subtask); }
	YY_BREAK
case 107:
YY_RULE_SETUP
{ put_byte (212, subtask); }
	YY_BREAK
case 108:
YY_RULE_SETUP
{ put_byte (213, subtask); }
	YY_BREAK
case 109:
YY_RULE_SETUP
{ put_byte (214, subtask); }
	YY_BREAK
case 110:
YY_RULE_SETUP
{ put_byte (216, subtask); }
	YY_BREAK
case 111:
YY_RULE_SETUP
{ put_byte (217, subtask); }
	YY_BREAK
case 112:
YY_RULE_SETUP
{ put_byte (218, subtask); }
	YY_BREAK
case 113:
YY_RULE_SETUP
{ put_byte (219, subtask); }
	YY_BREAK
case 114:
YY_RULE_SETUP
{ put_byte (220, subtask); }
	YY_BREAK
case 115:
YY_RULE_SETUP
{ put_byte (221, subtask); }
	YY_BREAK
case 116:
YY_RULE_SETUP
{ put_byte (223, subtask); }
	YY_BREAK
case 117:
YY_RULE_SETUP
{ put_byte (224, subtask); }
	YY_BREAK
case 118:
YY_RULE_SETUP
{ put_byte (225, subtask); }
	YY_BREAK
case 119:
YY_RULE_SETUP
{ put_byte (226, subtask); }
	YY_BREAK
case 120:
YY_RULE_SETUP
{ put_byte (227, subtask); }
	YY_BREAK
case 121:
YY_RULE_SETUP
{ put_byte (228, subtask); }
	YY_BREAK
case 122:
YY_RULE_SETUP
{ put_byte (229, subtask); }
	YY_BREAK
case 123:
YY_RULE_SETUP
{ put_byte (230, subtask); }
	YY_BREAK
case 124:
YY_RULE_SETUP
{ put_byte (231, subtask); }
	YY_BREAK
case 125:
YY_RULE_SETUP
{ put_byte (232, subtask); }
	YY_BREAK
case 126:
YY_RULE_SETUP
{ put_byte (233, subtask); }
	YY_BREAK
case 127:
YY_RULE_SETUP
{ put_byte (234, subtask); }
	YY_BREAK
case 128:
YY_RULE_SETUP
{ put_byte (235, subtask); }
	YY_BREAK
case 129:
YY_RULE_SETUP
{ put_byte (236, subtask); }
	YY_BREAK
case 130:
YY_RULE_SETUP
{ put_byte (237, subtask); }
	YY_BREAK
case 131:
YY_RULE_SETUP
{ put_byte (238, subtask); }
	YY_BREAK
case 132:
YY_RULE_SETUP
{ put_byte (239, subtask); }
	YY_BREAK
case 133:
YY_RULE_SETUP
{ put_byte (241, subtask); }
	YY_BREAK
case 134:
YY_RULE_SETUP
{ put_byte (242, subtask); }
	YY_BREAK
case 135:
YY_RULE_SETUP
{ put_byte (243, subtask); }
	YY_BREAK
case 136:
YY_RULE_SETUP
{ put_byte (244, subtask); }
	YY_BREAK
case 137:
YY_RULE_SETUP
{ put_byte (245, subtask); }
	YY_BREAK
case 138:
YY_RULE_SETUP
{ put_byte (246, subtask); }
	YY_BREAK
case 139:
YY_RULE_SETUP
{ put_byte (248, subtask); }
	YY_BREAK
case 140:
YY_RULE_SETUP
{ put_byte (249, subtask); }
	YY_BREAK
case 141:
YY_RULE_SETUP
{ put_byte (250, subtask); }
	YY_BREAK
case 142:
YY_RULE_SETUP
{ put_byte (251, subtask); }
	YY_BREAK
case 143:
YY_RULE_SETUP
{ put_byte (252, subtask); }
	YY_BREAK
case 144:
YY_RULE_SETUP
{ put_byte (253, subtask); }
	YY_BREAK
case 145:
YY_RULE_SETUP
{ put_byte (255, subtask); }
	YY_BREAK
case 146:
YY_RULE_SETUP
{ ECHO; }
	YY_BREAK
case 147:
YY_RULE_SETUP
{ ECHO; }
	YY_BREAK


case 148:
YY_RULE_SETUP
{ put_byte (171, subtask); }
	YY_BREAK
case 149:
YY_RULE_SETUP
{ put_byte (187, subtask); }
	YY_BREAK
case 150:
YY_RULE_SETUP
{ put_byte (192, subtask); }
	YY_BREAK
case 151:
YY_RULE_SETUP
{ put_byte (194, subtask); }
	YY_BREAK
case 152:
*yy_cp = yy_hold_char; /* undo effects of setting up yytext */
yy_c_buf_p = yy_cp = yy_bp + 2;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
{ if (yytext[1] == request->diaeresis_char)
			    put_byte (196, subtask);
			  else
			    ECHO;
			}
	YY_BREAK
case 153:
YY_RULE_SETUP
{ put_byte (224, subtask); }
	YY_BREAK
case 154:
YY_RULE_SETUP
{ put_byte (226, subtask); }
	YY_BREAK
case 155:
*yy_cp = yy_hold_char; /* undo effects of setting up yytext */
yy_c_buf_p = yy_cp = yy_bp + 2;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
{ if (yytext[1] == request->diaeresis_char)
			    put_byte (228, subtask);
			  else
			    ECHO;
			}
	YY_BREAK
case 156:
*yy_cp = yy_hold_char; /* undo effects of setting up yytext */
yy_c_buf_p = yy_cp = yy_bp + 2;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
{ put_byte (199, subtask); }
	YY_BREAK
case 157:
*yy_cp = yy_hold_char; /* undo effects of setting up yytext */
yy_c_buf_p = yy_cp = yy_bp + 2;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
{ put_byte (231, subtask); }
	YY_BREAK
case 158:
YY_RULE_SETUP
{ put_byte (200, subtask); }
	YY_BREAK
case 159:
YY_RULE_SETUP
{ put_byte (201, subtask); put_byte (187, subtask); }
	YY_BREAK
case 160:
YY_RULE_SETUP
{ put_byte ('E', subtask); put_byte (187, subtask); }
	YY_BREAK
case 161:
YY_RULE_SETUP
{ put_byte (201, subtask); }
	YY_BREAK
case 162:
YY_RULE_SETUP
{ put_byte (202, subtask); }
	YY_BREAK
case 163:
*yy_cp = yy_hold_char; /* undo effects of setting up yytext */
yy_c_buf_p = yy_cp = yy_bp + 2;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
{ if (yytext[1] == request->diaeresis_char)
			    put_byte (203, subtask);
			  else
			    ECHO;
			}
	YY_BREAK
case 164:
YY_RULE_SETUP
{ put_byte (232, subtask); }
	YY_BREAK
case 165:
YY_RULE_SETUP
{ put_byte (233, subtask); put_byte (187, subtask); }
	YY_BREAK
case 166:
YY_RULE_SETUP
{ put_byte ('e', subtask); put_byte (187, subtask); }
	YY_BREAK
case 167:
YY_RULE_SETUP
{ put_byte (233, subtask); }
	YY_BREAK
case 168:
YY_RULE_SETUP
{ put_byte (234, subtask); }
	YY_BREAK
case 169:
*yy_cp = yy_hold_char; /* undo effects of setting up yytext */
yy_c_buf_p = yy_cp = yy_bp + 2;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
{ if (yytext[1] == request->diaeresis_char)
			    put_byte (235, subtask);
			  else
			    ECHO;
			}
	YY_BREAK
case 170:
YY_RULE_SETUP
{ put_byte (204, subtask); }
	YY_BREAK
case 171:
YY_RULE_SETUP
{ put_byte (206, subtask); }
	YY_BREAK
case 172:
*yy_cp = yy_hold_char; /* undo effects of setting up yytext */
yy_c_buf_p = yy_cp = yy_bp + 2;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
{ if (yytext[1] == request->diaeresis_char)
			    put_byte (207, subtask);
			  else
			    ECHO;
			}
	YY_BREAK
case 173:
YY_RULE_SETUP
{ put_byte (236, subtask); }
	YY_BREAK
case 174:
YY_RULE_SETUP
{ put_byte (238, subtask); }
	YY_BREAK
case 175:
*yy_cp = yy_hold_char; /* undo effects of setting up yytext */
yy_c_buf_p = yy_cp = yy_bp + 2;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
{ if (yytext[1] == request->diaeresis_char)
			    put_byte (239, subtask);
			  else
			    ECHO;
			}
	YY_BREAK
case 176:
YY_RULE_SETUP
{ put_byte (210, subtask); }
	YY_BREAK
case 177:
YY_RULE_SETUP
{ put_byte (212, subtask); }
	YY_BREAK
case 178:
*yy_cp = yy_hold_char; /* undo effects of setting up yytext */
yy_c_buf_p = yy_cp = yy_bp + 2;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
{ if (yytext[1] == request->diaeresis_char)
			    put_byte (214, subtask);
			  else
			    ECHO;
			}
	YY_BREAK
case 179:
YY_RULE_SETUP
{ put_byte (242, subtask); }
	YY_BREAK
case 180:
YY_RULE_SETUP
{ put_byte (244, subtask); }
	YY_BREAK
case 181:
*yy_cp = yy_hold_char; /* undo effects of setting up yytext */
yy_c_buf_p = yy_cp = yy_bp + 2;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
{ if (yytext[1] == request->diaeresis_char)
			    put_byte (246, subtask);
			  else
			    ECHO;
			}
	YY_BREAK
case 182:
YY_RULE_SETUP
{ put_byte (217, subtask); }
	YY_BREAK
case 183:
YY_RULE_SETUP
{ put_byte (219, subtask); }
	YY_BREAK
case 184:
*yy_cp = yy_hold_char; /* undo effects of setting up yytext */
yy_c_buf_p = yy_cp = yy_bp + 2;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
{ if (yytext[1] == request->diaeresis_char)
			    put_byte (220, subtask);
			  else
			    ECHO;
			}
	YY_BREAK
case 185:
YY_RULE_SETUP
{ put_byte (249, subtask); }
	YY_BREAK
case 186:
YY_RULE_SETUP
{ put_byte (251, subtask); }
	YY_BREAK
case 187:
*yy_cp = yy_hold_char; /* undo effects of setting up yytext */
yy_c_buf_p = yy_cp = yy_bp + 2;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
{ if (yytext[1] == request->diaeresis_char)
			    put_byte (252, subtask);
			  else
			    ECHO;
			}
	YY_BREAK
case 188:
YY_RULE_SETUP
{ texte_latin1_diaeresis (); }
	YY_BREAK
case 189:
YY_RULE_SETUP
{ texte_latin1_diaeresis (); }
	YY_BREAK
case 190:
YY_RULE_SETUP
{ texte_latin1_diaeresis (); }
	YY_BREAK
case 191:
YY_RULE_SETUP
{ texte_latin1_diaeresis (); }
	YY_BREAK
case 192:
YY_RULE_SETUP
{ texte_latin1_diaeresis (); }
	YY_BREAK
case 193:
YY_RULE_SETUP
{ texte_latin1_diaeresis (); }
	YY_BREAK
case 194:
YY_RULE_SETUP
{ texte_latin1_diaeresis (); }
	YY_BREAK
case 195:
YY_RULE_SETUP
{ texte_latin1_diaeresis (); }
	YY_BREAK
case 196:
YY_RULE_SETUP
{ texte_latin1_diaeresis (); }
	YY_BREAK
case 197:
YY_RULE_SETUP
{ texte_latin1_diaeresis (); }
	YY_BREAK
case 198:
YY_RULE_SETUP
{ texte_latin1_diaeresis (); }
	YY_BREAK
case 199:
YY_RULE_SETUP
{ if (yytext[4] == request->diaeresis_char)
			    texte_latin1_diaeresis ();
			  else
			    ECHO;
			}
	YY_BREAK
case 200:
YY_RULE_SETUP
{ texte_latin1_diaeresis (); }
	YY_BREAK
case 201:
YY_RULE_SETUP
{ ECHO; }
	YY_BREAK
case 202:
YY_RULE_SETUP
{ texte_latin1_diaeresis (); }
	YY_BREAK
case 203:
YY_RULE_SETUP
{ texte_latin1_diaeresis (); }
	YY_BREAK
case 204:
YY_RULE_SETUP
{ texte_latin1_diaeresis (); }
	YY_BREAK
case 205:
YY_RULE_SETUP
{ texte_latin1_diaeresis (); }
	YY_BREAK
case 206:
YY_RULE_SETUP
{ texte_latin1_diaeresis (); }
	YY_BREAK
case 207:
YY_RULE_SETUP
{ texte_latin1_diaeresis (); }
	YY_BREAK

case 208:
YY_RULE_SETUP
ECHO;
	YY_BREAK

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - yytext_ptr) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = yy_hold_char;
		YY_RESTORE_YY_MORE_OFFSET

		if ( yy_current_buffer->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed yyin at a new source and called
			 * yylex().  If so, then we have to assure
			 * consistency between yy_current_buffer and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			yy_n_chars = yy_current_buffer->yy_n_chars;
			yy_current_buffer->yy_input_file = yyin;
			yy_current_buffer->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( yy_c_buf_p <= &yy_current_buffer->yy_ch_buf[yy_n_chars] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			yy_c_buf_p = yytext_ptr + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state();

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = yytext_ptr + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++yy_c_buf_p;
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = yy_c_buf_p;
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer() )
			{
			case EOB_ACT_END_OF_FILE:
				{
				yy_did_buffer_switch_on_eof = 0;

				if ( yywrap() )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					yy_c_buf_p = yytext_ptr + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				yy_c_buf_p =
					yytext_ptr + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state();

				yy_cp = yy_c_buf_p;
				yy_bp = yytext_ptr + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				yy_c_buf_p =
				&yy_current_buffer->yy_ch_buf[yy_n_chars];

				yy_current_state = yy_get_previous_state();

				yy_cp = yy_c_buf_p;
				yy_bp = yytext_ptr + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
	} /* end of yylex */


/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */

static int yy_get_next_buffer()
	{
	register char *dest = yy_current_buffer->yy_ch_buf;
	register char *source = yytext_ptr;
	register int number_to_move, i;
	int ret_val;

	if ( yy_c_buf_p > &yy_current_buffer->yy_ch_buf[yy_n_chars + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( yy_current_buffer->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( yy_c_buf_p - yytext_ptr - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) (yy_c_buf_p - yytext_ptr) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( yy_current_buffer->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		yy_current_buffer->yy_n_chars = yy_n_chars = 0;

	else
		{
		int num_to_read =
			yy_current_buffer->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */
#ifdef YY_USES_REJECT
			YY_FATAL_ERROR(
"input buffer overflow, can't enlarge buffer because scanner uses REJECT" );
#else

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = yy_current_buffer;

			int yy_c_buf_p_offset =
				(int) (yy_c_buf_p - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					yy_flex_realloc( (void *) b->yy_ch_buf,
							 b->yy_buf_size + 2 );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = 0;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			yy_c_buf_p = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = yy_current_buffer->yy_buf_size -
						number_to_move - 1;
#endif
			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&yy_current_buffer->yy_ch_buf[number_to_move]),
			yy_n_chars, num_to_read );

		yy_current_buffer->yy_n_chars = yy_n_chars;
		}

	if ( yy_n_chars == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			yyrestart( yyin );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			yy_current_buffer->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	yy_n_chars += number_to_move;
	yy_current_buffer->yy_ch_buf[yy_n_chars] = YY_END_OF_BUFFER_CHAR;
	yy_current_buffer->yy_ch_buf[yy_n_chars + 1] = YY_END_OF_BUFFER_CHAR;

	yytext_ptr = &yy_current_buffer->yy_ch_buf[0];

	return ret_val;
	}


/* yy_get_previous_state - get the state just before the EOB char was reached */

static yy_state_type yy_get_previous_state()
	{
	register yy_state_type yy_current_state;
	register char *yy_cp;

	yy_current_state = yy_start;

	for ( yy_cp = yytext_ptr + YY_MORE_ADJ; yy_cp < yy_c_buf_p; ++yy_cp )
		{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			yy_last_accepting_state = yy_current_state;
			yy_last_accepting_cpos = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 598 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		}

	return yy_current_state;
	}


/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */

#ifdef YY_USE_PROTOS
static yy_state_type yy_try_NUL_trans( yy_state_type yy_current_state )
#else
static yy_state_type yy_try_NUL_trans( yy_current_state )
yy_state_type yy_current_state;
#endif
	{
	register int yy_is_jam;
	register char *yy_cp = yy_c_buf_p;

	register YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		yy_last_accepting_state = yy_current_state;
		yy_last_accepting_cpos = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 598 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 597);

	return yy_is_jam ? 0 : yy_current_state;
	}


#ifndef YY_NO_UNPUT
#ifdef YY_USE_PROTOS
static void yyunput( int c, register char *yy_bp )
#else
static void yyunput( c, yy_bp )
int c;
register char *yy_bp;
#endif
	{
	register char *yy_cp = yy_c_buf_p;

	/* undo effects of setting up yytext */
	*yy_cp = yy_hold_char;

	if ( yy_cp < yy_current_buffer->yy_ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		register int number_to_move = yy_n_chars + 2;
		register char *dest = &yy_current_buffer->yy_ch_buf[
					yy_current_buffer->yy_buf_size + 2];
		register char *source =
				&yy_current_buffer->yy_ch_buf[number_to_move];

		while ( source > yy_current_buffer->yy_ch_buf )
			*--dest = *--source;

		yy_cp += (int) (dest - source);
		yy_bp += (int) (dest - source);
		yy_current_buffer->yy_n_chars =
			yy_n_chars = yy_current_buffer->yy_buf_size;

		if ( yy_cp < yy_current_buffer->yy_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	*--yy_cp = (char) c;


	yytext_ptr = yy_bp;
	yy_hold_char = *yy_cp;
	yy_c_buf_p = yy_cp;
	}
#endif	/* ifndef YY_NO_UNPUT */


#ifdef __cplusplus
static int yyinput()
#else
static int input()
#endif
	{
	int c;

	*yy_c_buf_p = yy_hold_char;

	if ( *yy_c_buf_p == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( yy_c_buf_p < &yy_current_buffer->yy_ch_buf[yy_n_chars] )
			/* This was really a NUL. */
			*yy_c_buf_p = '\0';

		else
			{ /* need more input */
			int offset = yy_c_buf_p - yytext_ptr;
			++yy_c_buf_p;

			switch ( yy_get_next_buffer() )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					yyrestart( yyin );

					/* fall through */

				case EOB_ACT_END_OF_FILE:
					{
					if ( yywrap() )
						return EOF;

					if ( ! yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					yy_c_buf_p = yytext_ptr + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) yy_c_buf_p;	/* cast for 8-bit char's */
	*yy_c_buf_p = '\0';	/* preserve yytext */
	yy_hold_char = *++yy_c_buf_p;


	return c;
	}


#ifdef YY_USE_PROTOS
void yyrestart( FILE *input_file )
#else
void yyrestart( input_file )
FILE *input_file;
#endif
	{
	if ( ! yy_current_buffer )
		yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE );

	yy_init_buffer( yy_current_buffer, input_file );
	yy_load_buffer_state();
	}


#ifdef YY_USE_PROTOS
void yy_switch_to_buffer( YY_BUFFER_STATE new_buffer )
#else
void yy_switch_to_buffer( new_buffer )
YY_BUFFER_STATE new_buffer;
#endif
	{
	if ( yy_current_buffer == new_buffer )
		return;

	if ( yy_current_buffer )
		{
		/* Flush out information for old buffer. */
		*yy_c_buf_p = yy_hold_char;
		yy_current_buffer->yy_buf_pos = yy_c_buf_p;
		yy_current_buffer->yy_n_chars = yy_n_chars;
		}

	yy_current_buffer = new_buffer;
	yy_load_buffer_state();

	/* We don't actually know whether we did this switch during
	 * EOF (yywrap()) processing, but the only time this flag
	 * is looked at is after yywrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	yy_did_buffer_switch_on_eof = 1;
	}


#ifdef YY_USE_PROTOS
void yy_load_buffer_state( void )
#else
void yy_load_buffer_state()
#endif
	{
	yy_n_chars = yy_current_buffer->yy_n_chars;
	yytext_ptr = yy_c_buf_p = yy_current_buffer->yy_buf_pos;
	yyin = yy_current_buffer->yy_input_file;
	yy_hold_char = *yy_c_buf_p;
	}


#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_create_buffer( FILE *file, int size )
#else
YY_BUFFER_STATE yy_create_buffer( file, size )
FILE *file;
int size;
#endif
	{
	YY_BUFFER_STATE b;

	b = (YY_BUFFER_STATE) yy_flex_alloc( sizeof( struct yy_buffer_state ) );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) yy_flex_alloc( b->yy_buf_size + 2 );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_is_our_buffer = 1;

	yy_init_buffer( b, file );

	return b;
	}


#ifdef YY_USE_PROTOS
void yy_delete_buffer( YY_BUFFER_STATE b )
#else
void yy_delete_buffer( b )
YY_BUFFER_STATE b;
#endif
	{
	if ( ! b )
		return;

	if ( b == yy_current_buffer )
		yy_current_buffer = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		yy_flex_free( (void *) b->yy_ch_buf );

	yy_flex_free( (void *) b );
	}


#ifndef YY_ALWAYS_INTERACTIVE
#ifndef YY_NEVER_INTERACTIVE
extern int isatty YY_PROTO(( int ));
#endif
#endif

#ifdef YY_USE_PROTOS
void yy_init_buffer( YY_BUFFER_STATE b, FILE *file )
#else
void yy_init_buffer( b, file )
YY_BUFFER_STATE b;
FILE *file;
#endif


	{
	yy_flush_buffer( b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

#if YY_ALWAYS_INTERACTIVE
	b->yy_is_interactive = 1;
#else
#if YY_NEVER_INTERACTIVE
	b->yy_is_interactive = 0;
#else
	b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
#endif
#endif
	}


#ifdef YY_USE_PROTOS
void yy_flush_buffer( YY_BUFFER_STATE b )
#else
void yy_flush_buffer( b )
YY_BUFFER_STATE b;
#endif

	{
	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == yy_current_buffer )
		yy_load_buffer_state();
	}


#ifndef YY_NO_SCAN_BUFFER
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_scan_buffer( char *base, yy_size_t size )
#else
YY_BUFFER_STATE yy_scan_buffer( base, size )
char *base;
yy_size_t size;
#endif
	{
	YY_BUFFER_STATE b;

	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) yy_flex_alloc( sizeof( struct yy_buffer_state ) );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_buffer()" );

	b->yy_buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = 0;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	yy_switch_to_buffer( b );

	return b;
	}
#endif


#ifndef YY_NO_SCAN_STRING
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_scan_string( yyconst char *yy_str )
#else
YY_BUFFER_STATE yy_scan_string( yy_str )
yyconst char *yy_str;
#endif
	{
	int len;
	for ( len = 0; yy_str[len]; ++len )
		;

	return yy_scan_bytes( yy_str, len );
	}
#endif


#ifndef YY_NO_SCAN_BYTES
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_scan_bytes( yyconst char *bytes, int len )
#else
YY_BUFFER_STATE yy_scan_bytes( bytes, len )
yyconst char *bytes;
int len;
#endif
	{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;

	/* Get memory for full buffer, including space for trailing EOB's. */
	n = len + 2;
	buf = (char *) yy_flex_alloc( n );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_bytes()" );

	for ( i = 0; i < len; ++i )
		buf[i] = bytes[i];

	buf[len] = buf[len+1] = YY_END_OF_BUFFER_CHAR;

	b = yy_scan_buffer( buf, n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in yy_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
	}
#endif


#ifndef YY_NO_PUSH_STATE
#ifdef YY_USE_PROTOS
static void yy_push_state( int new_state )
#else
static void yy_push_state( new_state )
int new_state;
#endif
	{
	if ( yy_start_stack_ptr >= yy_start_stack_depth )
		{
		yy_size_t new_size;

		yy_start_stack_depth += YY_START_STACK_INCR;
		new_size = yy_start_stack_depth * sizeof( int );

		if ( ! yy_start_stack )
			yy_start_stack = (int *) yy_flex_alloc( new_size );

		else
			yy_start_stack = (int *) yy_flex_realloc(
					(void *) yy_start_stack, new_size );

		if ( ! yy_start_stack )
			YY_FATAL_ERROR(
			"out of memory expanding start-condition stack" );
		}

	yy_start_stack[yy_start_stack_ptr++] = YY_START;

	BEGIN(new_state);
	}
#endif


#ifndef YY_NO_POP_STATE
static void yy_pop_state()
	{
	if ( --yy_start_stack_ptr < 0 )
		YY_FATAL_ERROR( "start-condition stack underflow" );

	BEGIN(yy_start_stack[yy_start_stack_ptr]);
	}
#endif


#ifndef YY_NO_TOP_STATE
static int yy_top_state()
	{
	return yy_start_stack[yy_start_stack_ptr - 1];
	}
#endif

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

#ifdef YY_USE_PROTOS
static void yy_fatal_error( yyconst char msg[] )
#else
static void yy_fatal_error( msg )
char msg[];
#endif
	{
	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
	}



/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
		yytext[yyleng] = yy_hold_char; \
		yy_c_buf_p = yytext + n; \
		yy_hold_char = *yy_c_buf_p; \
		*yy_c_buf_p = '\0'; \
		yyleng = n; \
		} \
	while ( 0 )


/* Internal utility routines. */

#ifndef yytext_ptr
#ifdef YY_USE_PROTOS
static void yy_flex_strncpy( char *s1, yyconst char *s2, int n )
#else
static void yy_flex_strncpy( s1, s2, n )
char *s1;
yyconst char *s2;
int n;
#endif
	{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
	}
#endif

#ifdef YY_NEED_STRLEN
#ifdef YY_USE_PROTOS
static int yy_flex_strlen( yyconst char *s )
#else
static int yy_flex_strlen( s )
yyconst char *s;
#endif
	{
	register int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
	}
#endif


#ifdef YY_USE_PROTOS
static void *yy_flex_alloc( yy_size_t size )
#else
static void *yy_flex_alloc( size )
yy_size_t size;
#endif
	{
	return (void *) malloc( size );
	}

#ifdef YY_USE_PROTOS
static void *yy_flex_realloc( void *ptr, yy_size_t size )
#else
static void *yy_flex_realloc( ptr, size )
void *ptr;
yy_size_t size;
#endif
	{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
	}

#ifdef YY_USE_PROTOS
static void yy_flex_free( void *ptr )
#else
static void yy_flex_free( ptr )
void *ptr;
#endif
	{
	free( ptr );
	}

#if YY_MAIN
int main()
	{
	yylex();
	return 0;
	}
#endif

/* Conversion of files between different charsets and surfaces.
   Copyright © 1990, 93, 94, 97, 98, 99, 00 Free Software Foundation, Inc.
   This file is part of the GNU C Library.
   Contributed by François Pinard <pinard@iro.umontreal.ca>, 1988.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License
   as published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the `recode' Library; see the file `COPYING.LIB'.
   If not, write to the Free Software Foundation, Inc., 59 Temple Place -
   Suite 330, Boston, MA 02111-1307, USA.  */


static bool
transform_ascii_latin1 (RECODE_SUBTASK subtask_argument)
{
  subtask = subtask_argument;
  request = subtask->task->request;
  yy_init = 1;
  yyin = subtask->input.file;
  yyout = subtask->output.file;
  BEGIN ascii_latin1;
  return yylex ();
}

bool
module_ascii_latin1 (RECODE_OUTER outer)
{
  if (!declare_single (outer, "ASCII-BS", "Latin-1",
		       outer->quality_variable_to_byte,
		       NULL, transform_ascii_latin1))
    return false;

  return true;
}

void
delmodule_ascii_latin1 (RECODE_OUTER outer)
{
}
/* Conversion of files between different charsets and surfaces.
   Copyright © 1990, 93, 97, 98, 99, 00 Free Software Foundation, Inc.
   Contributed by François Pinard <pinard@iro.umontreal.ca>, 1988.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License
   as published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the `recode' Library; see the file `COPYING.LIB'.
   If not, write to the Free Software Foundation, Inc., 59 Temple Place -
   Suite 330, Boston, MA 02111-1307, USA.  */


static bool
transform_latex_latin1 (RECODE_SUBTASK subtask_argument)
{
  subtask = subtask_argument;
  request = subtask->task->request;
  yy_init = 1;
  yyin = subtask->input.file;
  yyout = subtask->output.file;
  BEGIN latex_latin1;
  return yylex ();
}

bool
module_latex_latin1 (RECODE_OUTER outer)
{
  return
    declare_single (outer, "LaTeX", "Latin-1",
                    outer->quality_variable_to_byte,
                    NULL, transform_latex_latin1)
    && declare_alias (outer, "TeX", "LaTeX")
    && declare_alias (outer, "ltex", "LaTeX");
}

void
delmodule_latex_latin1 (RECODE_OUTER outer)
{
}
/* Conversion of files between different charsets and surfaces.
   Copyright © 1990, 93, 94, 97, 98, 99, 00 Free Software Foundation, Inc.
   Contributed by François Pinard <pinard@iro.umontreal.ca>, 1989.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License
   as published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the `recode' Library; see the file `COPYING.LIB'.
   If not, write to the Free Software Foundation, Inc., 59 Temple Place -
   Suite 330, Boston, MA 02111-1307, USA.  */


static bool
transform_texte_latin1 (RECODE_SUBTASK subtask_argument)
{
  subtask = subtask_argument;
  request = subtask->task->request;
  yy_init = 1;
  yyin = subtask->input.file;
  yyout = subtask->output.file;
  BEGIN texte_latin1;
  return yylex ();
}

void
texte_latin1_diaeresis (void)
{
  unsigned counter;

  for (counter = 0; counter < yyleng; counter++)
    if (yytext[counter+1] == request->diaeresis_char)
      {
	switch (yytext[counter])
	  {
	    /* The next "case 'A'" line once triggered a `NULL in input'
	       diagnostic in flex.  This astonishing bug has been hard to
	       isolate, so I'll leave this comment around for a while.  */

	  case 'A': put_byte (196, subtask); break;
	  case 'E': put_byte (203, subtask); break;
	  case 'I': put_byte (207, subtask); break;
	  case 'O': put_byte (214, subtask); break;
	  case 'U': put_byte (220, subtask); break;
	  case 'a': put_byte (228, subtask); break;
	  case 'e': put_byte (235, subtask); break;
	  case 'i': put_byte (239, subtask); break;
	  case 'o': put_byte (246, subtask); break;
	  case 'u': put_byte (252, subtask); break;
	  case 'y': put_byte (255, subtask); break;
	  default:  put_byte (yytext[counter], subtask);
	  }
	counter++;
      }
    else
      put_byte (yytext[counter], subtask);
}

bool
module_texte_latin1 (RECODE_OUTER outer)
{
  return
    declare_single (outer, "Texte", "Latin-1",
		    outer->quality_variable_to_byte, NULL,
		    transform_texte_latin1)
    && declare_alias (outer, "txte", "Texte");
}

void
delmodule_texte_latin1 (RECODE_OUTER outer)
{
}
