/* Conversion of files between different charsets and surfaces.
   Copyright © 1998, 99, 00 Free Software Foundation, Inc.
   François Pinard <pinard@iro.umontreal.ca>, 1998.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*===================================================================\
| This module is merely a sand box for now, and is not operational.  |
\===================================================================*/

#include "common.h"

/* FIXME: From charset.c.  */
/* Hash table size for charset names.  */
#define HASH_TABLE_SIZE 997

/* This file takes care of dynamically loading supplementary tables in source
   form, or later dumping them as compilable C code.  */

void
recode_freeze_tables (RECODE_OUTER outer)
{
#if 0
  /* Produce header block comment.  */

  fputs ("\
/* DO NOT MODIFY THIS FILE!  It was generated by `recode --freeze-tables'.  */\n\
\n\
/* Conversion of files between different charsets and surfaces.\n\
   Copyright © 1998 Free Software Foundation, Inc.\n\
   Contributed by François Pinard <pinard@iro.umontreal.ca>, 1998.\n\
\n\
   This library is free software; you can redistribute it and/or\n\
   modify it under the terms of the GNU Lesser General Public License\n\
   as published by the Free Software Foundation; either version 2 of the\n\
   License, or (at your option) any later version.\n\
\n\
   This library is distributed in the hope that it will be\n\
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty\n\
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\n\
   Lesser General Public License for more details.\n\
\n\
   You should have received a copy of the GNU Lesser General Public\n\
   License along with the `recode' Library; see the file `COPYING.LIB'.\n\
   If not, write to the Free Software Foundation, Inc., 59 Temple Place -\n\
   Suite 330, Boston, MA 02111-1307, USA.  */\n\
\n\
#include \"common.h\"\n",
	 stdout);
#endif

#if 0
  /* Save the hash table.  */

  {
    struct recode_hash *hash;

    fputs ("\n", stdout);
    printf ("struct recode_hash hash_table[%d] =\n", HASH_TABLE_SIZE);
    fputs ("  {\n", stdout);

    for (hash = outer->hash_table;
	 hash < outer->hash_table + HASH_TABLE_SIZE;
	 hash++)
      {
	fputs ("    {", stdout);
	if (hash->name)
	  printf ("\"%s\"", hash->name);
	else
	  fputs ("NULL", stdout);
	fputs (", CHARSET", stdout);
	fputs (", ", stdout);
	if (hash->next)
	  printf ("NEXT", hash->next);
	else
	  fputs ("NULL", stdout);
	fputs ("},\n", stdout);
      }
    fputs ("  };\n", stdout);
  }
#endif
}
